import { IOpenable } from 'pip-services3-commons-nodex';
import { IConfigurable } from 'pip-services3-commons-nodex';
import { IReferenceable } from 'pip-services3-commons-nodex';
import { IReferences } from 'pip-services3-commons-nodex';
import { ConfigParams } from 'pip-services3-commons-nodex';
import { CompositeLogger } from 'pip-services3-components-nodex';
import { CompositeCounters } from 'pip-services3-components-nodex';
import { DependencyResolver } from 'pip-services3-commons-nodex';
import { FilterParams } from 'pip-services3-commons-nodex';
import { PagingParams } from 'pip-services3-commons-nodex';
import { DataPage } from 'pip-services3-commons-nodex';
import { AwsConnectionResolver } from 'pip-services3-aws-nodex';
import { AwsConnectionParams } from 'pip-services3-aws-nodex';
import { BlobInfoV1 } from '../data/version1/BlobInfoV1';
import { IBlobsPersistence } from './IBlobsPersistence';
import { TempBlobStorage } from './TempBlobStorage';
export declare class BlobsS3Persistence implements IOpenable, IConfigurable, IReferenceable, IBlobsPersistence {
    private static readonly _defaultConfig;
    protected _s3: any;
    protected _opened: boolean;
    protected _connection: AwsConnectionParams;
    protected _bucket: string;
    protected _connectTimeout: number;
    protected _minChunkSize: number;
    protected _maxBlobSize: number;
    protected _reducedRedundancy: boolean;
    protected _maxPageSize: number;
    protected _dependencyResolver: DependencyResolver;
    protected _connectionResolver: AwsConnectionResolver;
    protected _logger: CompositeLogger;
    protected _counters: CompositeCounters;
    protected _storage: TempBlobStorage;
    configure(config: ConfigParams): void;
    setReferences(references: IReferences): void;
    isOpen(): boolean;
    open(correlationId: string): Promise<void>;
    close(correlationId: string): Promise<void>;
    private dataToInfo;
    private encodeString;
    private decodeString;
    private matchString;
    private matchSearch;
    private composeFilter;
    getPageByFilter(correlationId: string, filter: FilterParams, paging: PagingParams): Promise<DataPage<BlobInfoV1>>;
    getListByIds(correlationId: string, ids: string[]): Promise<BlobInfoV1[]>;
    getOneById(correlationId: string, id: string): Promise<BlobInfoV1>;
    update(correlationId: string, item: BlobInfoV1): Promise<BlobInfoV1>;
    markCompleted(correlationId: string, ids: string[]): Promise<void>;
    isUriSupported(): boolean;
    getUri(correlationId: string, id: string): Promise<string>;
    beginWrite(correlationId: string, item: BlobInfoV1): Promise<string>;
    private uploadPart;
    private uploadAndDeleteChunks;
    writeChunk(correlationId: string, token: string, chunk: string): Promise<string>;
    endWrite(correlationId: string, token: string, chunk: string): Promise<BlobInfoV1>;
    abortWrite(correlationId: string, token: string): Promise<void>;
    beginRead(correlationId: string, id: string): Promise<BlobInfoV1>;
    readChunk(correlationId: string, id: string, skip: number, take: number): Promise<string>;
    endRead(correlationId: string, id: string): Promise<void>;
    deleteById(correlationId: string, id: string): Promise<BlobInfoV1>;
    deleteByIds(correlationId: string, ids: string[]): Promise<void>;
    clear(correlationId: string): Promise<void>;
}
